pub enum BuildError {
NonemptyRootlessPath,
PathStartsWithDoubleSlash,
FirstPathSegmentContainsColon,
}Available on crate feature
alloc only.Expand description
An error occurred when building a URI/IRI (reference).
Variants
NonemptyRootlessPath
Authority is present, but the path is not empty and does not start with '/'.
PathStartsWithDoubleSlash
Authority is not present, but the path starts with "//".
FirstPathSegmentContainsColon
Neither scheme nor authority is present, but the first path segment contains ':'.